"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Listener = (function () {
    function Listener(store) {
        this.store = store;
        this.listeners = {
            mutationListeners: {},
            beforeActionListeners: {},
            afterActionListeners: {}
        };
        this.actionsAreListened = false;
        this.mutationsAreListened = false;
        this.addMutationListener = this.createSubscriber('mutationListeners', this.subscribeToMutations.bind(this));
        this.addBeforeActionListener = this.createSubscriber('beforeActionListeners', this.subscribeToActions.bind(this));
        this.addAfterActionListener = this.createSubscriber('afterActionListeners', this.subscribeToActions.bind(this));
    }
    Listener.prototype.subscribeToMutations = function () {
        var _this = this;
        if (!this.mutationsAreListened) {
            this.store.subscribe(function (mutation, state) {
                if (_this.listeners.mutationListeners[mutation.type] === undefined)
                    return;
                _this.listeners.mutationListeners[mutation.type].forEach(function (listener) { return listener(mutation.payload, state); });
            });
            this.mutationsAreListened = true;
        }
    };
    Listener.prototype.subscribeToActions = function () {
        var _this = this;
        if (!this.actionsAreListened) {
            this.store.subscribeAction({
                before: function (action, state) {
                    if (_this.listeners.beforeActionListeners[action.type] === undefined)
                        return;
                    _this.listeners.beforeActionListeners[action.type].forEach(function (listener) { return listener(action.payload, state); });
                },
                after: function (action, state) {
                    if (_this.listeners.afterActionListeners[action.type] === undefined)
                        return;
                    _this.listeners.afterActionListeners[action.type].forEach(function (listener) { return listener(action.payload, state); });
                }
            });
            this.actionsAreListened = true;
        }
    };
    Listener.prototype.createSubscriber = function (listeners, subscribe) {
        var _this = this;
        return function (key, listener) {
            subscribe();
            if (_this.listeners[listeners][key] === undefined) {
                _this.listeners[listeners][key] = [];
            }
            _this.listeners[listeners][key].push(listener);
            return function () {
                _this.listeners[listeners][key] = _this.listeners[listeners][key].filter(function (l) { return l !== listener; });
            };
        };
    };
    return Listener;
}());
exports.Listener = Listener;
exports.VuexListener = {
    install: function (Vue) {
        Vue.mixin({
            beforeCreate: function () {
                var options = this.$options;
                if (options.store) {
                    this.$storeListener = new Listener(options.store);
                }
                else if (options.parent && options.parent.$storeListener) {
                    this.$storeListener = options.parent.$storeListener;
                }
            }
        });
    }
};
